/*
**    TP CPE Lyon
**    Copyright (C) 2014 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/



#include "vec2.hpp"
#include "../libcommon/exception_cpe.hpp"

#include <cmath>
#include <sstream>

#include <assert.h>

namespace cpe
{

vec2::vec2():internal_x(0),internal_y(0){}
vec2::vec2(float x_param,float y_param):internal_x(x_param),internal_y(y_param){}
float vec2::x() const
{
    return internal_x;
}

float& vec2::x()
{
    return internal_x;
}

float vec2::y() const
{
    return internal_y;
}

float& vec2::y()
{
    return internal_y;
}


float vec2::operator[](const size_t& k) const
{
    assert_size(k);
    switch (k)
    {
    case 0:
        return internal_x;
    case 1:
        return internal_y;
    default:
        assert(false);
        throw cpe::exception_cpe("Probleme",MACRO_EXCEPTION_PARAMETER);
    };
}

float& vec2::operator[](const size_t& k)
{
    assert_size(k);
    switch (k)
    {
    case 0:
        return internal_x;
    case 1:
        return internal_y;
    default:
        assert(false);
        throw cpe::exception_cpe("Probleme",MACRO_EXCEPTION_PARAMETER);
    };
}

float vec2::operator()(const size_t& k) const
{
    return (*this)[k];
}
float& vec2::operator()(const size_t& k)
{
    return (*this)[k];
}

void vec2::set_zero()
{
    internal_x=0;
    internal_y=0;
}

float vec2::dot(const vec2& p) const
{
    return internal_x*p.x()+internal_y*p.y();
}

float vec2::norm() const
{
    return std::sqrt(norm2());
}

float vec2::norm2() const
{
    return internal_x*internal_x+internal_y*internal_y;
}

vec2 vec2::operator+(const vec2& p2) const
{
    return vec2(x()+p2.x(),y()+p2.y());
}

vec2 operator+(float s,const vec2& p)
{
    return vec2(s+p.x(),s+p.y());
}

vec2 vec2::operator+(float s) const
{
    vec2 temp=*this;
    temp+=s;
    return temp;
}

vec2 vec2::operator-(const vec2& p2) const
{
    return vec2(x()-p2.x(),y()-p2.y());
}

vec2 operator-(float s,const vec2& p)
{
    return vec2(s-p.x(),s-p.y());
}

vec2 vec2::operator-(float s) const
{
    return vec2(x()-s,y()-s);
}

vec2 operator*(float s,const vec2& p)
{
    return vec2(p.x()*s,p.y()*s);
}

vec2 vec2::operator*(float s) const
{
    return vec2(s*x(),s*y());
}

vec2 vec2::operator/(float s) const
{
    float epsilon=1e-8;
    if(std::abs(s)<epsilon)
        throw cpe::exception_cpe("Division par 0",MACRO_EXCEPTION_PARAMETER);

    return vec2(x()/s,y()/s);
}

vec2& vec2::operator+=(const vec2& p)
{
    internal_x+=p.x();
    internal_y+=p.y();
    return *this;
}

vec2& vec2::operator+=(float s)
{
    internal_x+=s;
    internal_y+=s;
    return *this;
}

vec2& vec2::operator-=(const vec2& p)
{
    internal_x-=p.x();
    internal_y-=p.y();
    return *this;
}

vec2& vec2::operator-=(float s)
{
    internal_x-=s;
    internal_y-=s;
    return *this;
}

vec2& vec2::operator*=(float s)
{
    internal_x*=s;
    internal_y*=s;
    return *this;
}

vec2& vec2::operator/=(float s)
{
    internal_x/=s;
    internal_y/=s;
    return *this;
}

vec2 vec2::operator-() const
{
    return vec2(-internal_x,-internal_y);
}

vec2 vec2::product_compontentwise(const vec2& p) const
{
    return vec2(internal_x*p.x(),internal_y*p.y());
}

vec2& vec2::product_compontentwise_internal(const vec2& p)
{
    internal_x*=p.x();
    internal_y*=p.y();
    return *this;
}
std::string vec2::to_string() const
{
    std::stringstream stream;
    stream<<internal_x<<" "<<internal_y;
    return stream.str();
}
std::ostream& operator<<(std::ostream& stream,const vec2& p)
{
    stream<<"("<<p.to_string()<<")";
    return stream;
}

void vec2::assert_size(const size_t& k) const
{
    if(k>1)
    {
        std::stringstream stream;
        stream<<"vec2::assert_size("<<k<<") k>2";
        throw exception_cpe(stream.str(),MACRO_EXCEPTION_PARAMETER);
    }
}
vec2 vec2::normalized() const
{
    float epsilon=1e-6;

    float n=norm();
    if(n<epsilon)
        throw exception_cpe("norm=0",MACRO_EXCEPTION_PARAMETER);

    return *this/n;
}
void vec2::scale(float sx,float sy)
{
    internal_x*=sx;
    internal_y*=sy;
}

const float *vec2::pointer() const
{return &internal_x;}
}
